' DOSROCKS.BAS
' by jenneygames.com

'-- core
DECLARE SUB init ()
DECLARE SUB controls ()
DECLARE SUB logic ()
DECLARE SUB render ()
DECLARE SUB shutdown ()

'-- draw routines
DECLARE SUB rRock ()
DECLARE SUB rStarship ()
DECLARE SUB rMissile ()

'-- specific routines
DECLARE FUNCTION testMissileRockCollision (m%, r%)
DECLARE FUNCTION testShipRockCollision (r%)
DECLARE SUB nextLevel ()
DECLARE SUB delay (s!)

'-- constants
CONST PI = 3.141592654#
CONST startingShips% = 3
CONST rockSpeedStart! = .3, rockSpeedIncrease! = .02
CONST shipRadius% = 20, shipRotationSpeed! = 90 / 16, thrustPower! = .25
CONST missileSpeed! = 25

'-- user types
TYPE shipType
  x AS SINGLE
  y AS SINGLE
  radius AS INTEGER
  shipMoveX AS SINGLE
  shipMoveY AS SINGLE
  rotation AS SINGLE
  shipOn AS INTEGER
  shipsLeft AS INTEGER
  shieldsLeft AS INTEGER
  ShieldsOn AS INTEGER
END TYPE

TYPE rockType
  x AS SINGLE
  y AS SINGLE
  rockMoveX AS SINGLE
  rockMoveY AS SINGLE
  rotation AS SINGLE
  rotationDirection AS INTEGER
  radius AS INTEGER
  rockOn AS INTEGER
END TYPE

TYPE missileType
  x AS SINGLE
  y AS SINGLE
  radius AS INTEGER
  missileMoveX AS SINGLE
  missileMoveY AS SINGLE
  missileOn AS INTEGER
END TYPE

TYPE controlType
  left AS INTEGER
  right AS INTEGER
  thrust AS INTEGER
  fire AS INTEGER
  shield AS INTEGER
  cancel AS INTEGER
END TYPE

TYPE gameTimerType
  startTime AS INTEGER
  lastTime AS SINGLE
  delayTimer AS INTEGER
  shieldTimer AS INTEGER
END TYPE

TYPE gameCoreType
  gameMode AS INTEGER
  gameScore AS INTEGER
  gameLevel AS INTEGER
  gameScoreLife AS INTEGER
END TYPE

'-- global variables
COMMON SHARED ship AS shipType
COMMON SHARED rock() AS rockType
COMMON SHARED missle() AS missileType
COMMON SHARED control AS controlType
COMMON SHARED gameTimer AS gameTimerType
COMMON SHARED gameCore AS gameCoreType
COMMON SHARED missile() AS missileType

'-- game loop
init
WHILE gameCore.gameMode > 0
  controls
  logic
  render
WEND
shutdown

SUB controls

 '-- get key press/release of key scan code via computer i/o
 k% = INP(&H60)

 '-- left
 IF k% = 75 THEN
   control.left = 1
   control.right = 0
   'control.thrust = 0
   'control.fire = 0
 END IF

 IF k% = 203 THEN
   control.left = 0
   control.right = 0
   control.thrust = 0
   control.fire = 0
 END IF

 '-- thrust
 IF k% = 72 THEN
   'control.left = 0
   'control.right = 0
   control.thrust = 1
   'control.fire = 0
 END IF

 IF k% = 200 THEN
   'control.left = 0
   'control.right = 0
   control.thrust = 0
   'control.fire = 0
 END IF

 '-- right
 IF k% = 77 THEN
   control.left = 0
   control.right = 1
   'control.thrust = 0
   'control.fire = 0
 END IF

 IF k% = 205 THEN
   control.left = 0
   control.right = 0
   'control.thrust = 0
   'control.fire = 0
 END IF
 
 '-- spacebar
 IF k% = 57 THEN
   control.left = 0
   control.right = 0
   control.thrust = 0
   control.fire = 1
 END IF

 IF k% = 185 THEN
   control.left = 0
   control.right = 0
   control.thrust = 0
   control.fire = 0
 END IF

 '-- left shift
 IF k% = 42 THEN
   control.shield = 1
 END IF

 IF k% = 170 THEN
   control.shield = 0
 END IF

 '-- right shift
 IF k% = 54 THEN
   control.shield = 1
 END IF

 IF k% = 182 THEN
   control.shield = 0
 END IF

 '-- esc
 IF k% = 1 THEN
   gameCore.gameMode = 0
 END IF

 '-- debug
 'LOCATE 5, 5: PRINT USING "######"; k%

 '-- discharge keyboard buffer, saturation and beeping
 DEF SEG = &H40
 POKE &H1A, PEEK(&H1C)

END SUB

SUB delay (s!)
 endtime! = TIMER + s!
 IF endtime! > 86400 THEN endtime! = endtime! - 86400

 DO
 LOOP UNTIL TIMER >= endtime! AND (TIMER - endtime!) < 1
END SUB

SUB init
 '
 '-- astroids.bas
 '-- spacer8c.bas
 '-- hairball.bas <-- 2nd place
 '-- dosrocks.bas <-- winner!
 '
 ' by jenneygames.com
 '

 gameCore.gameMode = 2
 gameCore.gameLevel = 0
 gameCore.gameScore = 0

 gameTimer.startTime = 0

 ship.shipsLeft = 3
 ship.shipOn = 1
 ship.ShieldsOn = 0
 ship.shieldsLeft = 3

 ship.x = 160
 ship.y = 100
 ship.rotation = 270
 ship.radius = 4

 RANDOMIZE TIMER

 '-- resize arrays
 REDIM missile(1 TO 5) AS missileType

 '-- title screen
 SCREEN 0
 COLOR 3, 0

 SOUND 200, 1
 SOUND 250, 1
 SOUND 200, 1

 FOR i% = 1 TO 1000
  LOCATE INT(RND * 24) + 1, INT(RND * 80) + 1: PRINT "."
 NEXT i%

 '-- game loaded music
 PLAY "MB L8 O2 CC EE GG CC"

  LOCATE 11, 30: PRINT "       "
  LOCATE 12, 30: PRINT "            "
  LOCATE 12, 30: PRINT "            "
  LOCATE 13, 30: PRINT "            "
  LOCATE 14, 30: PRINT "            "
  LOCATE 15, 30: PRINT "       "
  LOCATE 17, 30: PRINT "         ROCKS"
                                               ' DOS
  LOCATE 23, 60: PRINT "by jenneygames.com"

 delay (3)

 gameCore.gameMode = 1
 nextLevel

 '-- clear display and set video pages
 SCREEN 7, 0, 0, 1

END SUB

SUB logic

 '-- ship wrap to screen
 IF ship.y < 0 THEN ship.y = 200
 IF ship.y > 200 THEN ship.y = 0
 IF ship.x < 0 THEN ship.x = 320
 IF ship.x > 320 THEN ship.x = 0

 '-- ship rotation
 IF control.left = 1 THEN
   ship.rotation = ship.rotation + 1 * shipRotationSpeed
 END IF

 IF control.right = 1 THEN
   ship.rotation = ship.rotation - 1 * shipRotationSpeed
 END IF

 IF ship.rotation > 360 THEN ship.rotation = 0
 IF ship.rotation < 0 THEN ship.rotation = 360

 '-- debug ship rotation
 'LOCATE 15, 15: PRINT ship.rotation

 '-- ship thrust
 IF control.thrust = 1 THEN
   ship.shipMoveX = ship.shipMoveX - COS(ship.rotation * (PI / 180)) * thrustPower!
   ship.shipMoveY = ship.shipMoveY + SIN(ship.rotation * (PI / 180)) * thrustPower!
 ELSE
   IF ship.shipMoveX <> 0 THEN ship.shipMoveX = ship.shipMoveX * .975
   IF ship.shipMoveY <> 0 THEN ship.shipMoveY = ship.shipMoveY * .975
 END IF

 '-- update ship position
 ship.x = ship.x + ship.shipMoveX
 ship.y = ship.y + ship.shipMoveY

 '-- debug test screen wrap for ship
 'ship.y = ship.y + 4
 'ship.x = ship.x + 4

 '-- no more ships?
 IF ship.shipsLeft < 1 THEN
   gameCore.gameMode = 0
 END IF

 '-- no more rocks?
 loadNextLevel% = 1
 FOR i% = 1 TO 3 + gameCore.gameLevel
  IF rock(i%).rockOn = 1 THEN
    loadNextLevel% = 0
  END IF
 NEXT i%
 IF loadNextLevel% = 1 THEN nextLevel


 '-- update rock
 FOR i% = 1 TO 3 + gameCore.gameLevel
   IF rock(i%).rockOn = 1 THEN
     rock(i%).x = rock(i%).x + rock(i%).rockMoveX
     rock(i%).y = rock(i%).y + rock(i%).rockMoveY
    
     '-- rock wrap to screen
     IF rock(i%).y < 0 THEN rock(i%).y = 200
     IF rock(i%).y > 200 THEN rock(i%).y = 10
     IF rock(i%).x < 0 THEN rock(i%).x = 320
     IF rock(i%).x > 320 THEN rock(i%).x = 10
    
     rock(i%).rotation = rock(i%).rotation + rock(i%).rotationDirection
    
     IF rock(i%).rotation > 360 THEN rock(i%).rotation = 0
     IF rock(i%).rotation < 0 THEN rock(i%).rotation = 360
   END IF
 NEXT i%

 IF control.fire = 1 THEN
   FOR i% = 1 TO 4
     IF missile(i%).missileOn = 0 THEN
       '-- missle launched!
       missile(i%).missileOn = 1
       missile(i%).radius = 1
       missile(i%).missileMoveX = -COS(ship.rotation * (PI / 180)) * thrustPower!
       missile(i%).missileMoveY = SIN(ship.rotation * (PI / 180)) * thrustPower!
       missile(i%).x = ship.x + missile(i%).missileMoveX
       missile(i%).y = ship.y + missile(i%).missileMoveY

       '-- play sound: pew! pew!
       PLAY "MB L64 O2 C O1 F"
      
       '-- break
       EXIT FOR
     END IF
   NEXT i%
 END IF

 '-- move missiles
 FOR i% = 1 TO 5
   IF missile(i%).missileOn = 1 THEN
     missile(i%).x = missile(i%).x + missile(i%).missileMoveX * missileSpeed!
     missile(i%).y = missile(i%).y + missile(i%).missileMoveY * missileSpeed!

     IF missile(i%).y < 0 THEN missile(i%).missileOn = 0
     IF missile(i%).y > 200 THEN missile(i%).missileOn = 0
     IF missile(i%).x < 0 THEN missile(i%).missileOn = 0
     IF missile(i%).x > 320 THEN missile(i%).missileOn = 0

   END IF
 NEXT i%

 '-- collision check missile <==> rock
 'FOR m% = 1 TO 5
 '  IF missile(m%).missileOn = 1 THEN
 '     FOR r% = 1 TO 3 + gameCore.gameLevel
 '       IF rock(r%).rockOn = 1 THEN
 '         IF testMissileRockCollision(m%, r%) = 1 THEN
 '           rock(r%).rockOn = 0
 '           gameCore.gameScore = gameCore.gameScore + 100
 '         END IF
 '       END IF
 '     NEXT r%
 '  END IF
 'NEXT m%

FOR r% = 1 TO 3 + gameCore.gameLevel
  IF rock(r%).rockOn = 1 THEN
    FOR m% = 1 TO 5
      IF missile(m%).missileOn = 1 THEN
          IF testMissileRockCollision(m%, r%) = 1 THEN
            rock(r%).rockOn = 0
            gameCore.gameScore = gameCore.gameScore + 100
          END IF
        END IF
      NEXT m%
      IF ship.shipOn = 1 THEN
        IF testShipRockCollision(r%) = 1 THEN
        END IF
      END IF
   END IF
 NEXT r%

 '-- add extra life if score reaches every 10,000
 IF gameCore.gameScoreLife >= 10000 THEN
   gameCore.gameScoreLife = gameCore.gameScoreLife - 10000
   ship.shipsLeft = ship.shipsLeft + 1
   PLAY "MB L16 O4 C G O5 C"
 END IF
END SUB

SUB nextLevel
 gameCore.gameLevel = gameCore.gameLevel + 1

 REDIM rock(1 TO 3 + gameCore.gameLevel) AS rockType

 FOR i% = 1 TO 3 + gameCore.gameLevel
 
   rock(i%).x = INT(RND * 320) + 1
   rock(i%).y = 120 + (INT(RND * 2) - 2) * 80

   rock(i%).rockMoveX = (RND * 2) + -1
   rock(i%).rockMoveY = (RND * 2) + -1
   rock(i%).radius = (INT(RND * 3) + 1) * 10
   rock(i%).rotation = INT(RND * 360) + 1
   rock(i%).rotationDirection = (INT(RND * 2) * -2) + 1
   rock(i%).rockOn = 1

   '-- debug skip
   'i% = 3 + gameCore.gameLevel
 NEXT i%

 '-- play next level
 PLAY "MB L8 O2G O3C"

END SUB

SUB render

  '-- render
  rRock
  IF ship.shipOn = 1 THEN rStarship
  rMissile

  '-- gameScore
  ' LOCATE 2, 31: PRINT USING "####### "; gameCore.gameScore
  LOCATE 2, 2: PRINT gameCore.gameScore;
  LOCATE 2, 2: PRINT "";

  '-- debug: clock
  ' LOCATE 2, 30: PRINT TIME$; " "

  '-- shipsLeft
  LOCATE 23, 1
  FOR i% = 1 TO ship.shipsLeft
    LOCATE 23, i% + 1
    PRINT "";
    IF i% = 9 THEN EXIT FOR
  NEXT i%

  '-- shieldsLeft
  'LOCATE 23, 37: PRINT USING "#"; ship.shieldsLeft
  'LOCATE 23, 38: PRINT ""
 
  '-- shieldsLeft v2 (sorry, ran out time! next year? :)
  'y% = 39
  'FOR i% = 1 TO ship.shieldsLeft
  '  y% = y% - 1
  '  LOCATE 23, y%: PRINT "";
  'NEXT i%

  '-- instead, render level
  LOCATE 23, 37: PRINT USING "##"; gameCore.gameLevel

  '-- flip video page
  PCOPY 0, 1
  CLS
END SUB

SUB rMissile
  FOR i% = 1 TO 5
    IF missile(i%).missileOn = 1 THEN
      x% = INT(missile(i%).x)
      y% = INT(missile(i%).y)
      DRAW "B M" + STR$(x%) + "," + STR$(y%)
      DRAW "S2 C6 NU1 ND1 NL1 NR1"
    END IF
  NEXT i%
END SUB

SUB rRock
  FOR i% = 1 TO 3 + gameCore.gameLevel
    IF rock(i%).rockOn = 1 THEN
      x% = INT(rock(i%).x)
      y% = INT(rock(i%).y)
      r% = INT(rock(i%).rotation)
      DRAW "B M" + STR$(x%) + "," + STR$(y%) + "TA" + STR$(r%)
      'CIRCLE STEP(0, 0), rock(i%).radius
      DRAW "S" + STR$(rock(i%).radius) + " C5 BM-4,0 M+4,+3 M+4,-3 M-2,-3 M-4,0 M-2,+3"
      'DRAW "S" + STR$(rock(i%).radius) + " C5 BM+3,0 M-0,-2 M-6,+2 M+6,+2 M+0,-2 BM-2,+0 " 'P3,3"
    END IF
  NEXT i%
END SUB

SUB rStarship
'-- if not blown up
 x% = INT(ship.x)
 y% = INT(ship.y)
 DRAW "B M" + STR$(x%) + "," + STR$(y%) + "TA" + STR$(INT(ship.rotation))
' DRAW "S16 C3 M-2,4 M+4,0 M-2,-4 BM+0,+1 P3,3"
 DRAW "S8 C3 BM+3,0 M-0,-2 M-6,+2 M+6,+2 M+0,-2 BM-2,+0 P3,3"

END SUB

SUB shutdown
 IF ship.shipsLeft > 0 THEN
   '-- early exit
   PLAY "MB L20 O2 C O1 G C"
 ELSE
   '-- game over music
   PLAY "MB MS L6 O2 DDD L1 O1 B"
 END IF
 delay (1)
 SCREEN 2
 CLS
 PRINT "Thanks for playing DOSROCKS by jenneygames.com!"
 PRINT "You ended at level"; gameCore.gameLevel; "with a score of"; gameCore.gameScore; ""
 delay (2)
END SUB

FUNCTION testMissileRockCollision (m%, r%)
 ' rock(i%).x = 0 AND rock(i%).y = 0 AND rock(i%).size = 0
 ' AND missile(m%).x = 0 AND missile(m%).y = 0
 '
 ' LOCATE 15, 15
 ' PRINT rock(i%).x
 
 ' dist = SQR((tilex - circenterx)^2) + ((tiley - circentery)^2)
 '
 ' qb64sourcecode.com
 DIM sideA%, sideB%, hypot&
 missileRockCollision = 0

 IF missile(m%).x < rock(r%).x - rock(r%).radius OR missile(m%).x > rock(r%).x + rock(r%).radius THEN EXIT FUNCTION
 IF missile(m%).y < rock(r%).y - rock(r%).radius OR missile(m%).y > rock(r%).y + rock(r%).radius THEN EXIT FUNCTION

 sideA% = missile(m%).x - rock(r%).x
 sideB% = missile(m%).y - rock(r%).y
 hypot& = sideA% * sideA% + sideB% * sideB%
 IF hypot& <= (missile(m%).radius + rock(r%).radius) * (missile(m%).radius + rock(r%).radius) THEN
   missileRockCollision = 1
   rock(r%).rockOn = 0
   PLAY "MB ML L32 O1 C O0 A B"
   gameCore.gameScore = gameCore.gameScore + 100
   gameCore.gameScoreLife = gameCore.gameScoreLife + 100
   missile(m%).missileOn = 0
 END IF
END FUNCTION

FUNCTION testShipRockCollision (r%)
 DIM sideA%, sideB%, hypot&

 IF ship.x < rock(r%).x - rock(r%).radius OR ship.x > rock(r%).x + rock(r%).radius THEN EXIT FUNCTION
 IF ship.y < rock(r%).y - rock(r%).radius OR ship.y > rock(r%).y + rock(r%).radius THEN EXIT FUNCTION

 sideA% = ship.x - rock(r%).x
 sideB% = ship.y - rock(r%).y
 hypot& = sideA% * sideA% + sideB% * sideB%
 IF hypot& <= (ship.radius + rock(r%).radius) * (ship.radius + rock(r%).radius) THEN
   ship.shipOn = 0
   rock(r%).rockOn = 0
   ship.shipsLeft = ship.shipsLeft - 1
   ship.x = 160
   ship.y = 100
   ship.rotation = 270
   ship.shipMoveX = 0
   ship.shipMoveY = 0

   IF ship.shipsLeft > 0 THEN
     ship.shipOn = 1
     PLAY "MB MS L6 O2 D L2 O1 A"
   END IF
 
 END IF

END FUNCTION

